@@ -14,27 +14,34 @@
 import logging
 import os
 import re
+import subprocess
+import sys
 from argparse import Namespace
-from typing import Any, List, Optional
+from typing import Any, Optional
 
+import torch
 from lightning_utilities.core.imports import RequirementCache
 from typing_extensions import get_args
 
 from lightning.fabric.accelerators import CPUAccelerator, CUDAAccelerator, MPSAccelerator
 from lightning.fabric.plugins.precision.precision import _PRECISION_INPUT_STR, _PRECISION_INPUT_STR_ALIAS
 from lightning.fabric.strategies import STRATEGY_REGISTRY
+from lightning.fabric.utilities.consolidate_checkpoint import _process_cli_args
 from lightning.fabric.utilities.device_parser import _parse_gpu_ids
+from lightning.fabric.utilities.distributed import _suggested_max_num_threads
+from lightning.fabric.utilities.load import _load_distributed_checkpoint
 
 _log = logging.getLogger(__name__)
 
 _CLICK_AVAILABLE = RequirementCache("click")
+_LIGHTNING_SDK_AVAILABLE = RequirementCache("lightning_sdk")
 
 _SUPPORTED_ACCELERATORS = ("cpu", "gpu", "cuda", "mps", "tpu")
 
 
-def _get_supported_strategies() -> List[str]:
-    """Returns strategy choices from the registry, with the ones removed that are incompatible to be launched from
-    the CLI or ones that require further configuration by the user."""
+def _get_supported_strategies() -> list[str]:
+    """Returns strategy choices from the registry, with the ones removed that are incompatible to be launched from the
+    CLI or ones that require further configuration by the user."""
     available_strategies = STRATEGY_REGISTRY.available_strategies()
     excluded = r".*(spawn|fork|notebook|xla|tpu|offload).*"
     return [strategy for strategy in available_strategies if not re.match(excluded, strategy)]
@@ -43,11 +50,34 @@ def _get_supported_strategies() -> List[str]:
 if _CLICK_AVAILABLE:
     import click
 
-    @click.command(
-        "model",
-        context_settings=dict(
-            ignore_unknown_options=True,
-        ),
+    def _legacy_main() -> None:
+        """Legacy CLI handler for fabric.
+
+        Raises deprecation warning and runs through fabric cli if necessary, else runs the entrypoint directly
+
+        """
+        hparams = sys.argv[1:]
+        if len(hparams) >= 2 and hparams[0] == "run" and hparams[1] == "model":
+            print(
+                "`lightning run model` is deprecated and will be removed in future versions."
+                " Please call `fabric run` instead."
+            )
+            _main()
+            return
+
+        if _LIGHTNING_SDK_AVAILABLE:
+            subprocess.run([sys.executable, "-m", "lightning_sdk.cli.entrypoint"] + hparams)
+            return
+
+    @click.group()
+    def _main() -> None:
+        pass
+
+    @_main.command(
+        "run",
+        context_settings={
+            "ignore_unknown_options": True,
+        },
     )
     @click.argument(
         "script",
@@ -110,27 +140,60 @@ if _CLICK_AVAILABLE:
         type=click.Choice(get_args(_PRECISION_INPUT_STR) + get_args(_PRECISION_INPUT_STR_ALIAS)),
         default=None,
         help=(
-            "Double precision (``64-true`` or ``64``), full precision (``32-true`` or ``64``), "
+            "Double precision (``64-true`` or ``64``), full precision (``32-true`` or ``32``), "
             "half precision (``16-mixed`` or ``16``) or bfloat16 precision (``bf16-mixed`` or ``bf16``)"
         ),
     )
     @click.argument("script_args", nargs=-1, type=click.UNPROCESSED)
-    def _run_model(**kwargs: Any) -> None:
+    def _run(**kwargs: Any) -> None:
         """Run a Lightning Fabric script.
 
         SCRIPT is the path to the Python script with the code to run. The script must contain a Fabric object.
 
         SCRIPT_ARGS are the remaining arguments that you can pass to the script itself and are expected to be parsed
         there.
+
         """
         script_args = list(kwargs.pop("script_args", []))
         main(args=Namespace(**kwargs), script_args=script_args)
 
+    @_main.command(
+        "consolidate",
+        context_settings={
+            "ignore_unknown_options": True,
+        },
+    )
+    @click.argument(
+        "checkpoint_folder",
+        type=click.Path(exists=True),
+    )
+    @click.option(
+        "--output_file",
+        type=click.Path(exists=True),
+        default=None,
+        help=(
+            "Path to the file where the converted checkpoint should be saved. The file should not already exist."
+            " If no path is provided, the file will be saved next to the input checkpoint folder with the same name"
+            " and a '.consolidated' suffix."
+        ),
+    )
+    def _consolidate(checkpoint_folder: str, output_file: Optional[str]) -> None:
+        """Convert a distributed/sharded checkpoint into a single file that can be loaded with `torch.load()`.
+
+        Only supports FSDP sharded checkpoints at the moment.
+
+        """
+        args = Namespace(checkpoint_folder=checkpoint_folder, output_file=output_file)
+        config = _process_cli_args(args)
+        checkpoint = _load_distributed_checkpoint(config.checkpoint_folder)
+        torch.save(checkpoint, config.output_file)
+
 
 def _set_env_variables(args: Namespace) -> None:
     """Set the environment variables for the new processes.
 
     The Fabric connector will parse the arguments set here.
+
     """
     os.environ["LT_CLI_USED"] = "1"
     if args.accelerator is not None:
@@ -158,14 +221,11 @@ def _get_num_processes(accelerator: str, devices: str) -> int:
     return len(parsed_devices) if parsed_devices is not None else 0
 
 
-def _torchrun_launch(args: Namespace, script_args: List[str]) -> None:
+def _torchrun_launch(args: Namespace, script_args: list[str]) -> None:
     """This will invoke `torchrun` programmatically to launch the given script in new processes."""
     import torch.distributed.run as torchrun
 
-    if args.strategy == "dp":
-        num_processes = 1
-    else:
-        num_processes = _get_num_processes(args.accelerator, args.devices)
+    num_processes = 1 if args.strategy == "dp" else _get_num_processes(args.accelerator, args.devices)
 
     torchrun_args = [
         f"--nproc_per_node={num_processes}",
@@ -178,11 +238,11 @@ def _torchrun_launch(args: Namespace, script_args: List[str]) -> None:
     torchrun_args.extend(script_args)
 
     # set a good default number of threads for OMP to avoid warnings being emitted to the user
-    os.environ.setdefault("OMP_NUM_THREADS", str(max(1, (os.cpu_count() or 1) // num_processes)))
+    os.environ.setdefault("OMP_NUM_THREADS", str(_suggested_max_num_threads()))
     torchrun.main(torchrun_args)
 
 
-def main(args: Namespace, script_args: Optional[List[str]] = None) -> None:
+def main(args: Namespace, script_args: Optional[list[str]] = None) -> None:
     _set_env_variables(args)
     _torchrun_launch(args, script_args or [])
 
@@ -195,4 +255,4 @@ if __name__ == "__main__":
         )
         raise SystemExit(1)
 
-    _run_model()
+    _run()
