@@ -16,38 +16,37 @@ import json
 import logging
 import os
 import platform
-from contextlib import contextmanager
+from collections.abc import Mapping
+from contextlib import AbstractContextManager, ExitStack
+from datetime import timedelta
 from itertools import chain
 from pathlib import Path
-from typing import Any, Dict, Generator, List, Mapping, Optional, Tuple, TYPE_CHECKING, Union
+from typing import TYPE_CHECKING, Any, Callable, Optional, Union
 
 import torch
 from lightning_utilities.core.imports import RequirementCache
 from torch.nn import Module
 from torch.optim import Optimizer
+from typing_extensions import override
 
 from lightning.fabric.accelerators import Accelerator, CUDAAccelerator
+from lightning.fabric.plugins.collectives.torch_collective import default_pg_timeout
 from lightning.fabric.plugins.environments.cluster_environment import ClusterEnvironment
 from lightning.fabric.plugins.precision import Precision
 from lightning.fabric.strategies.ddp import DDPStrategy
+from lightning.fabric.strategies.registry import _StrategyRegistry
 from lightning.fabric.strategies.strategy import _Sharded
 from lightning.fabric.utilities.distributed import log
-from lightning.fabric.utilities.imports import _TORCH_GREATER_EQUAL_2_0
-from lightning.fabric.utilities.rank_zero import rank_zero_info, rank_zero_only, rank_zero_warn
+from lightning.fabric.utilities.load import _move_state_into
+from lightning.fabric.utilities.rank_zero import rank_zero_info, rank_zero_warn
 from lightning.fabric.utilities.seed import reset_seed
 from lightning.fabric.utilities.types import _PATH
 
-_DEEPSPEED_AVAILABLE = (
-    # DeepSpeed fails under 0.8.2 with torch 2.0: https://github.com/microsoft/DeepSpeed/pull/2863
-    RequirementCache("deepspeed>=0.8.2")
-    or not _TORCH_GREATER_EQUAL_2_0
-    # check packaging because of https://github.com/microsoft/DeepSpeed/pull/2771
-    # remove the packaging check when min version is >=0.8.1
-    and RequirementCache("deepspeed")
-    and RequirementCache("packaging>=20.0")
-)
-if TYPE_CHECKING and _DEEPSPEED_AVAILABLE:
-    import deepspeed
+if TYPE_CHECKING:
+    from deepspeed import DeepSpeedEngine
+
+_DEEPSPEED_AVAILABLE = RequirementCache("deepspeed")
+_DEEPSPEED_GREATER_EQUAL_0_14_1 = RequirementCache("deepspeed>=0.14.1")
 
 
 # TODO(fabric): Links in the docstrings to PL-specific deepspeed user docs need to be replaced.
@@ -59,7 +58,7 @@ class DeepSpeedStrategy(DDPStrategy, _Sharded):
         accelerator: Optional[Accelerator] = None,
         zero_optimization: bool = True,
         stage: int = 2,
-        remote_device: str = "cpu",
+        remote_device: Optional[str] = None,
         offload_optimizer: bool = False,
         offload_parameters: bool = False,
         offload_params_device: str = "cpu",
@@ -84,9 +83,9 @@ class DeepSpeedStrategy(DDPStrategy, _Sharded):
         reduce_bucket_size: int = 200_000_000,
         zero_allow_untested_optimizer: bool = True,
         logging_batch_size_per_gpu: Optional[int] = None,
-        config: Optional[Union[_PATH, Dict[str, Any]]] = None,
+        config: Optional[Union[_PATH, dict[str, Any]]] = None,
         logging_level: int = logging.WARN,
-        parallel_devices: Optional[List[torch.device]] = None,
+        parallel_devices: Optional[list[torch.device]] = None,
         cluster_environment: Optional[ClusterEnvironment] = None,
         loss_scale: float = 0,
         initial_scale_power: int = 16,
@@ -100,6 +99,7 @@ class DeepSpeedStrategy(DDPStrategy, _Sharded):
         load_full_weights: bool = False,
         precision: Optional[Precision] = None,
         process_group_backend: Optional[str] = None,
+        timeout: Optional[timedelta] = default_pg_timeout,
     ) -> None:
         """Provides capabilities to run training using the DeepSpeed library, with training optimizations for large
         billion parameter models. `For more information: https://pytorch-
@@ -120,7 +120,7 @@ class DeepSpeedStrategy(DDPStrategy, _Sharded):
                 1 is optimizer state partitioning, 2 is optimizer+gradient state partitioning,
                 3 is optimizer+gradient_parameter partitioning using the infinity engine.
 
-            remote_device: Device to instantiate the model on initially (``cpu`` or ``nvme``).
+            remote_device: Device to instantiate the model on initially (``cpu`` or ``nvme``). Defaults to GPU.
 
             offload_optimizer: Enable offloading optimizer memory and computation to CPU or NVMe
                 based on ``offload_optimizer_device``.
@@ -228,6 +228,7 @@ class DeepSpeedStrategy(DDPStrategy, _Sharded):
             load_full_weights: True when loading a single checkpoint file containing the model state dict
                 when using ZeRO Stage 3. This differs from the DeepSpeed checkpoint which contains shards
                 per worker.
+
         """
         if not _DEEPSPEED_AVAILABLE:
             raise ImportError(
@@ -243,6 +244,7 @@ class DeepSpeedStrategy(DDPStrategy, _Sharded):
             process_group_backend=process_group_backend,
         )
         self._backward_sync_control = None  # DeepSpeed handles gradient accumulation internally
+        self._timeout: Optional[timedelta] = timeout
 
         self.config = self._load_config(config)
         if self.config is None:
@@ -295,7 +297,7 @@ class DeepSpeedStrategy(DDPStrategy, _Sharded):
         self.hysteresis = hysteresis
         self.min_loss_scale = min_loss_scale
 
-        self._deepspeed_engine: Optional["deepspeed.DeepSpeedEngine"] = None
+        self._deepspeed_engine: Optional[DeepSpeedEngine] = None
 
     @property
     def zero_stage_3(self) -> bool:
@@ -304,16 +306,18 @@ class DeepSpeedStrategy(DDPStrategy, _Sharded):
         return zero_optimization is not None and zero_optimization.get("stage") == 3
 
     @property
-    def distributed_sampler_kwargs(self) -> Dict[str, int]:
-        return dict(num_replicas=self.world_size, rank=self.global_rank)
+    @override
+    def distributed_sampler_kwargs(self) -> dict[str, int]:
+        return {"num_replicas": self.world_size, "rank": self.global_rank}
 
     @property
-    def model(self) -> "deepspeed.DeepSpeedEngine":
+    def model(self) -> "DeepSpeedEngine":
         return self._deepspeed_engine
 
+    @override
     def setup_module_and_optimizers(
-        self, module: Module, optimizers: List[Optimizer]
-    ) -> Tuple["deepspeed.DeepSpeedEngine", List[Optimizer]]:
+        self, module: Module, optimizers: list[Optimizer]
+    ) -> tuple["DeepSpeedEngine", list[Optimizer]]:
         """Set up a model and multiple optimizers together.
 
         Currently, only a single optimizer is supported.
@@ -321,6 +325,7 @@ class DeepSpeedStrategy(DDPStrategy, _Sharded):
         Return:
             The model wrapped into a :class:`deepspeed.DeepSpeedEngine` and a list with a single
             deepspeed optimizer.
+
         """
         if len(optimizers) != 1:
             raise ValueError(
@@ -332,48 +337,59 @@ class DeepSpeedStrategy(DDPStrategy, _Sharded):
         self._set_deepspeed_activation_checkpointing()
         return self._deepspeed_engine, [optimizer]
 
-    def setup_module(self, module: Module) -> "deepspeed.DeepSpeedEngine":
+    @override
+    def setup_module(self, module: Module) -> "DeepSpeedEngine":
         """Set up a module for inference (no optimizers).
 
         For training, see :meth:`setup_module_and_optimizers`.
+
         """
         self._deepspeed_engine, _ = self._initialize_engine(module)
         return self._deepspeed_engine
 
+    @override
     def setup_optimizer(self, optimizer: Optimizer) -> Optimizer:
         """Optimizers can only be set up jointly with the model in this strategy.
 
         Please use :meth:`setup_module_and_optimizers` to set up both module and optimizer together.
+
         """
         raise NotImplementedError(self._err_msg_joint_setup_required())
 
-    @contextmanager
-    def module_sharded_context(self) -> Generator[None, None, None]:
+    @override
+    def module_init_context(self, empty_init: Optional[bool] = None) -> AbstractContextManager:
+        if self.zero_stage_3 and empty_init is False:
+            raise NotImplementedError(
+                f"`{empty_init=}` is not a valid choice with `DeepSpeedStrategy` when ZeRO stage 3 is enabled."
+            )
+        module_sharded_ctx = self.module_sharded_context()
+        stack = ExitStack()
+        if not self.zero_stage_3:
+            stack.enter_context(super().module_init_context(empty_init=empty_init))
+        stack.enter_context(module_sharded_ctx)
+        return stack
+
+    @override
+    def module_sharded_context(self) -> AbstractContextManager:
         # Current limitation in Fabric: The config needs to be fully determined at the time of calling the context
-        # manager, which happens at the start of `Fabric.run()`. Later modificatoins through e.g. `Fabric.setup()`
-        # won't have an effect here.
+        # manager. Later modifications through e.g. `Fabric.setup()` won't have an effect here.
 
         import deepspeed
 
-        if self.zero_stage_3:
-            assert self._config_initialized
-
-            if self.precision.precision == "16-mixed":
-                dtype = torch.float16
-            elif self.precision.precision == "bf16-mixed":
-                dtype = torch.bfloat16
-            else:
-                dtype = torch.float32
-
-            with deepspeed.zero.Init(
-                remote_device=self.remote_device, pin_memory=True, config_dict_or_path=self.config, dtype=dtype
-            ):
-                yield
-        else:
-            yield
+        assert self._config_initialized
+        return deepspeed.zero.Init(
+            enabled=self.zero_stage_3,
+            remote_device=self.remote_device,
+            config_dict_or_path=self.config,
+        )
 
+    @override
     def save_checkpoint(
-        self, path: _PATH, state: Dict[str, Union[Module, Optimizer, Any]], storage_options: Optional[Any] = None
+        self,
+        path: _PATH,
+        state: dict[str, Union[Module, Optimizer, Any]],
+        storage_options: Optional[Any] = None,
+        filter: Optional[dict[str, Callable[[str, Any], bool]]] = None,
     ) -> None:
         """Save model, optimizer, and other state in a checkpoint directory.
 
@@ -382,6 +398,7 @@ class DeepSpeedStrategy(DDPStrategy, _Sharded):
             state: A dictionary with contents to be saved. If the dict contains modules or optimizers, their
                 state-dict will be retrieved and converted automatically.
             storage_options: Unused by this strategy, since it doesn't use a ``CheckpointIO`` plugin.
+            filter: Unsupported.
 
         Raises:
             TypeError:
@@ -389,12 +406,18 @@ class DeepSpeedStrategy(DDPStrategy, _Sharded):
             ValueError:
                 When no :class:`deepspeed.DeepSpeedEngine` objects were found in the state, or when multiple
                 :class:`deepspeed.DeepSpeedEngine` objects were found.
+
         """
         if storage_options is not None:
             raise TypeError(
                 "`DeepSpeedStrategy.save_checkpoint(..., storage_options=...)` is not supported because"
                 " `DeepSpeedStrategy` does not use the `CheckpointIO`."
             )
+        if filter is not None:
+            raise TypeError(
+                "`DeepSpeedStrategy.save_checkpoint(..., filter=...)` is not supported because"
+                " `DeepSpeedStrategy` manages the state serialization internally."
+            )
 
         engines = _get_deepspeed_engines_from_state(state)
         if len(engines) == 0:
@@ -403,7 +426,7 @@ class DeepSpeedStrategy(DDPStrategy, _Sharded):
                 " part of the state like so: `save_checkpoint(..., state={'model': model, ...})`. Make sure"
                 " you set up the model (and optimizers if any) through the strategy before saving the checkpoint."
             )
-        elif len(engines) > 1:
+        if len(engines) > 1:
             raise ValueError(
                 "Found multiple DeepSpeed engine modules in the given state. Saving checkpoints with DeepSpeed is"
                 " currently limited to a single model per checkpoint. To save multiple models, call the"
@@ -421,19 +444,24 @@ class DeepSpeedStrategy(DDPStrategy, _Sharded):
         state = {k: v for k, v in state.items() if v not in excluded_objects}
         _validate_state_keys(state)
         # there might be other stateful objects unrelated to the deepspeed engine - convert them to a state_dict
-        state = self._convert_stateful_objects_in_state(state)
+        state = self._convert_stateful_objects_in_state(state, filter={})
         # use deepspeed's internal checkpointing function to handle partitioned weights across processes
         engine.save_checkpoint(path, client_state=state, tag="checkpoint")
 
+    @override
     def load_checkpoint(
-        self, path: _PATH, state: Optional[Dict[str, Union[Module, Optimizer, Any]]] = None
-    ) -> Dict[str, Any]:
+        self,
+        path: _PATH,
+        state: Optional[Union[Module, Optimizer, dict[str, Union[Module, Optimizer, Any]]]] = None,
+        strict: bool = True,
+    ) -> dict[str, Any]:
         """Load the contents from a checkpoint and restore the state of the given objects.
 
         Args:
             path: A path to where the file is located
             state: A dictionary of objects whose state will be restored in-place from the checkpoint path.
                 This should contain exactly one model, and the model must already be set up by DeepSpeed.
+            strict: Whether to enforce that the keys in `state` match the keys in the checkpoint.
 
         Returns:
             Dictionary with the state inside DeepSpeed's engine
@@ -445,12 +473,13 @@ class DeepSpeedStrategy(DDPStrategy, _Sharded):
             RuntimeError:
                 If DeepSpeed was unable to load the checkpoint due to missing files or because the checkpoint is
                 not in the expected DeepSpeed format.
+
         """
-        if self.load_full_weights and self.zero_stage_3:
+        if isinstance(state, (Module, Optimizer)) or self.load_full_weights and self.zero_stage_3:
             # This code path to enables loading a checkpoint from a non-deepspeed checkpoint or from
             # a consolidated checkpoint
             path = self.broadcast(path)
-            return super().load_checkpoint(path=path, state=state)
+            return super().load_checkpoint(path=path, state=state, strict=strict)
 
         if not state:
             raise ValueError(
@@ -458,6 +487,7 @@ class DeepSpeedStrategy(DDPStrategy, _Sharded):
                 f" a model instance to reload is required. Pass it in like so:"
                 " DeepSpeedStrategy.load_checkpoint(..., state={'model': model, ...})"
             )
+        _validate_checkpoint_directory(path)
 
         engines = _get_deepspeed_engines_from_state(state)
         if len(engines) == 0:
@@ -466,14 +496,20 @@ class DeepSpeedStrategy(DDPStrategy, _Sharded):
                 " part of the state like so: `load_checkpoint(..., state={'model': model, ...})`. Make sure"
                 " you set up the model (and optimizers if any) through the strategy before loading the checkpoint."
             )
-        elif len(engines) > 1:
+        if len(engines) > 1:
             raise ValueError(
                 "Found multiple DeepSpeed engine modules in the given state. Saving and loading checkpoints"
                 " with DeepSpeed is currently limited to a single model per checkpoint. To load multiple model"
                 " states, call the load method for each model checkpoint separately."
             )
         engine = engines[0]
-        optimzer_state_requested = bool(len([item for item in state.values() if isinstance(item, Optimizer)]))
+
+        if _DEEPSPEED_GREATER_EQUAL_0_14_1:
+            from deepspeed.runtime.base_optimizer import DeepSpeedOptimizer
+        else:
+            from deepspeed.runtime import DeepSpeedOptimizer
+
+        optimzer_state_requested = any(isinstance(item, (Optimizer, DeepSpeedOptimizer)) for item in state.values())
 
         torch.cuda.empty_cache()
         _, client_state = engine.load_checkpoint(
@@ -481,22 +517,25 @@ class DeepSpeedStrategy(DDPStrategy, _Sharded):
             tag="checkpoint",
             load_optimizer_states=optimzer_state_requested,
             load_lr_scheduler_states=False,
-            load_module_strict=True,  # TODO(fabric): make strict loading configurable
+            load_module_strict=strict,
         )
+
         if client_state is None:
             raise RuntimeError(
                 "DeepSpeed was unable to load the checkpoint. Ensure you passed in a DeepSpeed compatible checkpoint"
                 " or a single checkpoint file by setting `DeepSpeedStrategy(..., load_full_weights=True)`."
             )
-        for k, v in client_state.copy().items():
-            if k not in state:
-                continue
-            state[k] = client_state.pop(k)
+
+        # `Engine.load_checkpoint` adds useless keys 'optimizer' and 'lr_scheduler' to the client state; remove
+        # them to avoid name collision with user state
+        keys = set(client_state) & set(state) - {"optimizer", "lr_scheduler"}
+        _move_state_into(source=client_state, destination=state, keys=keys)
         return client_state
 
+    @override
     def clip_gradients_norm(
         self,
-        module: "deepspeed.DeepSpeedEngine",
+        module: "DeepSpeedEngine",
         optimizer: Optimizer,
         max_norm: Union[float, int],
         norm_type: Union[float, int] = 2.0,
@@ -507,8 +546,9 @@ class DeepSpeedStrategy(DDPStrategy, _Sharded):
             "Make sure to set the `gradient_clipping` value in your Config."
         )
 
+    @override
     def clip_gradients_value(
-        self, module: "deepspeed.DeepSpeedEngine", optimizer: Optimizer, clip_val: Union[float, int]
+        self, module: "DeepSpeedEngine", optimizer: Optimizer, clip_val: Union[float, int]
     ) -> None:
         raise NotImplementedError(
             "DeepSpeed handles gradient clipping automatically within the optimizer. "
@@ -516,9 +556,17 @@ class DeepSpeedStrategy(DDPStrategy, _Sharded):
         )
 
     @classmethod
-    def register_strategies(cls, strategy_registry: Dict) -> None:
+    @override
+    def register_strategies(cls, strategy_registry: _StrategyRegistry) -> None:
         strategy_registry.register("deepspeed", cls, description="Default DeepSpeed Strategy")
         strategy_registry.register("deepspeed_stage_1", cls, description="DeepSpeed with ZeRO Stage 1 enabled", stage=1)
+        strategy_registry.register(
+            "deepspeed_stage_1_offload",
+            cls,
+            description="DeepSpeed with ZeRO Stage 1 and optimizer CPU Offload",
+            stage=1,
+            offload_optimizer=True,
+        )
         strategy_registry.register("deepspeed_stage_2", cls, description="DeepSpeed with ZeRO Stage 2 enabled", stage=2)
         strategy_registry.register(
             "deepspeed_stage_2_offload",
@@ -552,10 +600,11 @@ class DeepSpeedStrategy(DDPStrategy, _Sharded):
         self,
         model: Module,
         optimizer: Optional[Optimizer] = None,
-    ) -> Tuple["deepspeed.DeepSpeedEngine", Optimizer]:
+    ) -> tuple["DeepSpeedEngine", Optimizer]:
         """Initialize one model and one optimizer with an optional learning rate scheduler.
 
-        This calls :func:`deepspeed.initialize` internally.
+        This calls ``deepspeed.initialize`` internally.
+
         """
         import deepspeed
 
@@ -570,15 +619,21 @@ class DeepSpeedStrategy(DDPStrategy, _Sharded):
         )
         return deepspeed_engine, deepspeed_optimizer
 
-    def _setup_distributed(self) -> None:
+    @override
+    def setup_environment(self) -> None:
         if not isinstance(self.accelerator, CUDAAccelerator):
             raise RuntimeError(
                 f"The DeepSpeed strategy is only supported on CUDA GPUs but `{self.accelerator.__class__.__name__}`"
                 " is used."
             )
+        super().setup_environment()
+
+    @override
+    def _setup_distributed(self) -> None:
+        assert self.parallel_devices is not None
+        _validate_device_index_selection(self.parallel_devices)
         reset_seed()
         self._set_world_ranks()
-        rank_zero_only.rank = self.global_rank
         self._init_deepspeed_distributed()
         if not self._config_initialized:
             self._format_config()
@@ -597,7 +652,9 @@ class DeepSpeedStrategy(DDPStrategy, _Sharded):
                 f"MEMBER: {self.global_rank + 1}/{self.world_size}"
             )
         self._process_group_backend = self._get_process_group_backend()
-        deepspeed.init_distributed(self._process_group_backend, distributed_port=self.cluster_environment.main_port)
+        deepspeed.init_distributed(
+            self._process_group_backend, distributed_port=self.cluster_environment.main_port, timeout=self._timeout
+        )
 
     def _set_node_environment_variables(self) -> None:
         assert self.cluster_environment is not None
@@ -629,25 +686,15 @@ class DeepSpeedStrategy(DDPStrategy, _Sharded):
             )
 
         self.config.setdefault("train_micro_batch_size_per_gpu", 1)
-        self._format_precision_config()
-
-    def _format_precision_config(self) -> None:
-        assert isinstance(self.config, dict)
-        if self.precision.precision == "16-mixed":
-            if "fp16" not in self.config:
-                # FP16 is a DeepSpeed standalone AMP implementation
-                rank_zero_info("Enabling DeepSpeed FP16.")
-                self.config["fp16"] = {
-                    "enabled": True,
-                    "loss_scale": self.loss_scale,
-                    "initial_scale_power": self.initial_scale_power,
-                    "loss_scale_window": self.loss_scale_window,
-                    "hysteresis": self.hysteresis,
-                    "min_loss_scale": self.min_loss_scale,
-                }
-        elif "bf16" not in self.config and self.precision.precision == "bf16-mixed":
-            rank_zero_info("Enabling DeepSpeed BF16.")
-            self.config["bf16"] = {"enabled": True}
+        _format_precision_config(
+            config=self.config,
+            precision=self.precision.precision,
+            loss_scale=self.loss_scale,
+            loss_scale_window=self.loss_scale_window,
+            min_loss_scale=self.min_loss_scale,
+            initial_scale_power=self.initial_scale_power,
+            hysteresis=self.hysteresis,
+        )
 
     def _create_default_config(
         self,
@@ -674,7 +721,7 @@ class DeepSpeedStrategy(DDPStrategy, _Sharded):
         overlap_events: bool,
         thread_count: int,
         **zero_kwargs: Any,
-    ) -> Dict:
+    ) -> dict:
         cfg = {
             "activation_checkpointing": {
                 "partition_activations": partition_activations,
@@ -709,30 +756,29 @@ class DeepSpeedStrategy(DDPStrategy, _Sharded):
                     "max_in_cpu": max_in_cpu,
                     "pin_memory": pin_memory,
                 }
-            cfg.update(
-                {
-                    "zero_allow_untested_optimizer": zero_allow_untested_optimizer,
-                    "zero_optimization": zero_config,
-                }
-            )
+            cfg.update({
+                "zero_allow_untested_optimizer": zero_allow_untested_optimizer,
+                "zero_optimization": zero_config,
+            })
         if logging_batch_size_per_gpu:
             cfg["train_micro_batch_size_per_gpu"] = logging_batch_size_per_gpu
         return cfg
 
     def _restore_zero_state(self, module: Module, ckpt: Mapping[str, Any]) -> None:
-        """Overrides the normal load_state_dict behaviour in PyTorch to ensure we gather parameters that may be
-        sharded across processes before loading the state dictionary when using ZeRO stage 3. This is then
-        automatically synced across processes.
+        """Overrides the normal load_state_dict behaviour in PyTorch to ensure we gather parameters that may be sharded
+        across processes before loading the state dictionary when using ZeRO stage 3. This is then automatically synced
+        across processes.
 
         Args:
             ckpt: The ckpt file.
+
         """
         import deepspeed
 
         def load(module: torch.nn.Module, prefix: str = "") -> None:
-            missing_keys: List[str] = []
-            unexpected_keys: List[str] = []
-            error_msgs: List[str] = []
+            missing_keys: list[str] = []
+            unexpected_keys: list[str] = []
+            error_msgs: list[str] = []
             state_dict = ckpt["state_dict"]
 
             # copy state_dict so _load_from_state_dict can modify it
@@ -763,7 +809,7 @@ class DeepSpeedStrategy(DDPStrategy, _Sharded):
 
         load(module, prefix="")
 
-    def _load_config(self, config: Optional[Union[_PATH, Dict[str, Any]]]) -> Optional[Dict[str, Any]]:
+    def _load_config(self, config: Optional[Union[_PATH, dict[str, Any]]]) -> Optional[dict[str, Any]]:
         if config is None and self.DEEPSPEED_ENV_VAR in os.environ:
             rank_zero_info(f"Loading DeepSpeed config from set {self.DEEPSPEED_ENV_VAR} environment variable")
             config = os.environ[self.DEEPSPEED_ENV_VAR]
@@ -778,15 +824,14 @@ class DeepSpeedStrategy(DDPStrategy, _Sharded):
         return config
 
 
-def _get_deepspeed_engines_from_state(state: Dict[str, Any]) -> List["deepspeed.DeepSpeedEngine"]:
+def _get_deepspeed_engines_from_state(state: dict[str, Any]) -> list["DeepSpeedEngine"]:
     from deepspeed import DeepSpeedEngine
 
     modules = chain(*(module.modules() for module in state.values() if isinstance(module, Module)))
-    engines = [engine for engine in modules if isinstance(engine, DeepSpeedEngine)]
-    return engines
+    return [engine for engine in modules if isinstance(engine, DeepSpeedEngine)]
 
 
-def _validate_state_keys(state: Dict[str, Any]) -> None:
+def _validate_state_keys(state: dict[str, Any]) -> None:
     # DeepSpeed merges the client state into its internal engine state when saving, but it does not check for
     # colliding keys from the user. We explicitly check it here:
     deepspeed_internal_keys = {
@@ -811,3 +856,79 @@ def _validate_state_keys(state: Dict[str, Any]) -> None:
             " values being overwritten by DeepSpeed. Consider changing the name of these keys to something else: "
             + ", ".join(colliding_keys)
         )
+
+
+def _validate_device_index_selection(parallel_devices: list[torch.device]) -> None:
+    selected_device_indices = [device.index for device in parallel_devices]
+    expected_device_indices = list(range(len(parallel_devices)))
+    if selected_device_indices != expected_device_indices:
+        raise RuntimeError(
+            f"The selected device indices {selected_device_indices!r} don't match the local rank values of processes."
+            " If you need to select GPUs at a specific index, set the `CUDA_VISIBLE_DEVICES` environment variable"
+            f" instead. For example: `CUDA_VISIBLE_DEVICES={','.join(str(i) for i in selected_device_indices)}`."
+        )
+
+
+def _is_deepspeed_checkpoint(path: Path) -> bool:
+    """Heuristic check whether the path points to a top-level DeepSpeed checkpoint directory."""
+    return path.is_dir() and (path / "checkpoint").is_dir()
+
+
+def _validate_checkpoint_directory(path: _PATH) -> None:
+    """Validates that the path points to a DeepSpeed checkpoint directory and suggests fixes for user error."""
+    # Example DeepSpeed checkpoint directory:
+    #
+    # epoch=5-step=10999.ckpt
+    # ├── checkpoint
+    # │   ├── zero_pp_rank_0_mp_rank_00_model_states.pt
+    # │   ├── zero_pp_rank_0_mp_rank_00_optim_states.pt
+    # │   ├── zero_pp_rank_1_mp_rank_00_model_states.pt
+    # │   └── zero_pp_rank_1_mp_rank_00_optim_states.pt
+    # ├── latest
+    # └── zero_to_fp32.py
+
+    path = Path(path)
+    path_is_ds_checkpoint = _is_deepspeed_checkpoint(path)
+    default_message = f"The provided path is not a valid DeepSpeed checkpoint: {path}"
+
+    if not path_is_ds_checkpoint:
+        # Case 1: User may have accidentally passed the subfolder "checkpoint"
+        parent_is_ds_checkpoint = _is_deepspeed_checkpoint(path.parent)
+        if parent_is_ds_checkpoint:
+            raise FileNotFoundError(
+                f"{default_message}. It looks like you passed the path to a subfolder."
+                f" Try to load using this parent directory instead: {path.parent}"
+            )
+        # Case 2: User may have accidentally passed the path to a file inside the "checkpoint" subfolder
+        parent_parent_is_ds_checkpoint = path.is_file() and _is_deepspeed_checkpoint(path.parent.parent)
+        if parent_parent_is_ds_checkpoint:
+            raise FileNotFoundError(
+                f"{default_message}. It looks like you passed the path to a file inside a DeepSpeed checkpoint folder."
+                f" Try to load using this parent directory instead: {path.parent.parent}"
+            )
+        raise FileNotFoundError(default_message)
+
+
+def _format_precision_config(
+    config: dict[str, Any],
+    precision: str,
+    loss_scale: float,
+    loss_scale_window: int,
+    min_loss_scale: int,
+    initial_scale_power: int,
+    hysteresis: int,
+) -> None:
+    if "fp16" not in config and precision in ("16-mixed", "16-true"):
+        # FP16 is a DeepSpeed standalone AMP implementation
+        rank_zero_info("Enabling DeepSpeed FP16. Model parameters and inputs will be cast to `float16`.")
+        config["fp16"] = {
+            "enabled": True,
+            "loss_scale": loss_scale,
+            "initial_scale_power": initial_scale_power,
+            "loss_scale_window": loss_scale_window,
+            "hysteresis": hysteresis,
+            "min_loss_scale": min_loss_scale,
+        }
+    elif "bf16" not in config and precision in ("bf16-mixed", "bf16-true"):
+        rank_zero_info("Enabling DeepSpeed BF16. Model parameters and inputs will be cast to `bfloat16`.")
+        config["bf16"] = {"enabled": True}
