@@ -12,11 +12,14 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 """Utilities related to data saving/loading."""
+
 import io
+import logging
 from pathlib import Path
-from typing import Any, Dict, IO, Union
+from typing import IO, Any, Union
 
 import fsspec
+import fsspec.utils
 import torch
 from fsspec.core import url_to_fs
 from fsspec.implementations.local import AbstractFileSystem
@@ -24,10 +27,13 @@ from lightning_utilities.core.imports import module_available
 
 from lightning.fabric.utilities.types import _MAP_LOCATION_TYPE, _PATH
 
+log = logging.getLogger(__name__)
+
 
 def _load(
     path_or_url: Union[IO, _PATH],
     map_location: _MAP_LOCATION_TYPE = None,
+    weights_only: bool = False,
 ) -> Any:
     """Loads a checkpoint.
 
@@ -41,15 +47,21 @@ def _load(
         return torch.load(
             path_or_url,
             map_location=map_location,  # type: ignore[arg-type] # upstream annotation is not correct
+            weights_only=weights_only,
         )
     if str(path_or_url).startswith("http"):
         return torch.hub.load_state_dict_from_url(
             str(path_or_url),
             map_location=map_location,  # type: ignore[arg-type]
+            weights_only=weights_only,
         )
     fs = get_filesystem(path_or_url)
     with fs.open(path_or_url, "rb") as f:
-        return torch.load(f, map_location=map_location)  # type: ignore[arg-type]
+        return torch.load(
+            f,
+            map_location=map_location,  # type: ignore[arg-type]
+            weights_only=weights_only,
+        )
 
 
 def get_filesystem(path: _PATH, **kwargs: Any) -> AbstractFileSystem:
@@ -57,7 +69,7 @@ def get_filesystem(path: _PATH, **kwargs: Any) -> AbstractFileSystem:
     return fs
 
 
-def _atomic_save(checkpoint: Dict[str, Any], filepath: Union[str, Path]) -> None:
+def _atomic_save(checkpoint: dict[str, Any], filepath: Union[str, Path]) -> None:
     """Saves a checkpoint atomically, avoiding the creation of incomplete checkpoints.
 
     Args:
@@ -69,8 +81,12 @@ def _atomic_save(checkpoint: Dict[str, Any], filepath: Union[str, Path]) -> None
 
     """
     bytesbuffer = io.BytesIO()
+    log.debug(f"Saving checkpoint: {filepath}")
     torch.save(checkpoint, bytesbuffer)
-    with fsspec.open(filepath, "wb") as f:
+
+    # We use a transaction here to avoid file corruption if the save gets interrupted
+    fs, urlpath = fsspec.core.url_to_fs(str(filepath))
+    with fs.transaction, fs.open(urlpath, "wb") as f:
         f.write(bytesbuffer.getvalue())
 
 
@@ -124,3 +140,7 @@ def _is_dir(fs: AbstractFileSystem, path: Union[str, Path], strict: bool = False
         return not fs.isfile(path)
 
     return fs.isdir(path)
+
+
+def _is_local_file_protocol(path: _PATH) -> bool:
+    return fsspec.utils.get_protocol(str(path)) == "file"
