@@ -15,16 +15,18 @@ r"""
 ModelPruning
 ^^^^^^^^^^^^
 """
+
 import inspect
 import logging
+from collections.abc import Sequence
 from copy import deepcopy
 from functools import partial
-from typing import Any, Callable, Dict, List, Optional, Sequence, Tuple, Union
+from typing import Any, Callable, Optional, Union
 
 import torch.nn.utils.prune as pytorch_prune
 from lightning_utilities.core.apply_func import apply_to_collection
-from torch import nn, Tensor
-from typing_extensions import TypedDict
+from torch import Tensor, nn
+from typing_extensions import TypedDict, override
 
 import lightning.pytorch as pl
 from lightning.pytorch.callbacks.callback import Callback
@@ -48,14 +50,14 @@ _PYTORCH_PRUNING_METHOD = {
     "random_unstructured": pytorch_prune.RandomUnstructured,
 }
 
-_PARAM_TUPLE = Tuple[nn.Module, str]
+_PARAM_TUPLE = tuple[nn.Module, str]
 _PARAM_LIST = Sequence[_PARAM_TUPLE]
 _MODULE_CONTAINERS = (LightningModule, nn.Sequential, nn.ModuleList, nn.ModuleDict)
 
 
 class _LayerRef(TypedDict):
     data: nn.Module
-    names: List[Tuple[int, str]]
+    names: list[tuple[int, str]]
 
 
 class ModelPruning(Callback):
@@ -65,7 +67,7 @@ class ModelPruning(Callback):
         self,
         pruning_fn: Union[Callable, str],
         parameters_to_prune: _PARAM_LIST = (),
-        parameter_names: Optional[List[str]] = None,
+        parameter_names: Optional[list[str]] = None,
         use_global_unstructured: bool = True,
         amount: Union[int, float, Callable[[int], Union[int, float]]] = 0.5,
         apply_pruning: Union[bool, Callable[[int], bool]] = True,
@@ -77,8 +79,8 @@ class ModelPruning(Callback):
         verbose: int = 0,
         prune_on_train_epoch_end: bool = True,
     ) -> None:
-        """Model pruning Callback, using PyTorch's prune utilities. This callback is responsible of pruning
-        networks parameters during training.
+        """Model pruning Callback, using PyTorch's prune utilities. This callback is responsible of pruning networks
+        parameters during training.
 
         To learn more about pruning with PyTorch, please take a look at
         `this tutorial <https://pytorch.org/tutorials/intermediate/pruning_tutorial.html>`_.
@@ -155,6 +157,7 @@ class ModelPruning(Callback):
                 if ``pruning_norm`` is not provided when ``"ln_structured"``,
                 if ``pruning_fn`` is neither ``str`` nor :class:`torch.nn.utils.prune.BasePruningMethod`, or
                 if ``amount`` is none of ``int``, ``float`` and ``Callable``.
+
         """
 
         self._use_global_unstructured = use_global_unstructured
@@ -163,8 +166,8 @@ class ModelPruning(Callback):
         self._resample_parameters = resample_parameters
         self._prune_on_train_epoch_end = prune_on_train_epoch_end
         self._parameter_names = parameter_names or self.PARAMETER_NAMES
-        self._global_kwargs: Dict[str, Any] = {}
-        self._original_layers: Optional[Dict[int, _LayerRef]] = None
+        self._global_kwargs: dict[str, Any] = {}
+        self._original_layers: Optional[dict[int, _LayerRef]] = None
         self._pruning_method_name: Optional[str] = None
 
         for name in self._parameter_names:
@@ -203,14 +206,14 @@ class ModelPruning(Callback):
             raise MisconfigurationException(
                 f"`pruning_fn` is expected to be a str in {list(_PYTORCH_PRUNING_FUNCTIONS.keys())}"
                 f" or a PyTorch `BasePruningMethod`. Found: {pruning_fn}."
-                " HINT: if passing a `BasePruningMethod`, pass the the class, not an instance"
+                " HINT: if passing a `BasePruningMethod`, pass the class, not an instance"
             )
 
         # need to ignore typing here since pytorch base class does not define the PRUNING_TYPE attribute
         if use_global_unstructured and pruning_fn.PRUNING_TYPE != "unstructured":  # type: ignore
             raise MisconfigurationException(
-                'Only the "unstructured" PRUNING_TYPE is supported with `use_global_unstructured=True`.'  # type: ignore
-                f" Found method {pruning_fn} of type {pruning_fn.PRUNING_TYPE}. "
+                'Only the "unstructured" PRUNING_TYPE is supported with `use_global_unstructured=True`.'
+                f" Found method {pruning_fn} of type {pruning_fn.PRUNING_TYPE}. "  # type: ignore[union-attr]
             )
 
         self.pruning_fn = pruning_fn
@@ -238,6 +241,7 @@ class ModelPruning(Callback):
 
         IF use_global_unstructured, pruning_fn will be resolved into its associated ``PyTorch BasePruningMethod`` ELSE,
         pruning_fn will be resolved into its function counterpart from `torch.nn.utils.prune`.
+
         """
         pruning_meth = (
             _PYTORCH_PRUNING_METHOD[pruning_fn]
@@ -262,6 +266,7 @@ class ModelPruning(Callback):
         """Removes pruning buffers from any pruned modules.
 
         Adapted from https://github.com/pytorch/pytorch/blob/v1.7.1/torch/nn/utils/prune.py#L1118-L1122
+
         """
         for _, module in module.named_modules():
             for k in list(module._forward_pre_hooks):
@@ -279,8 +284,7 @@ class ModelPruning(Callback):
         dst.data = src.data.to(dst.device)
 
     def apply_lottery_ticket_hypothesis(self) -> None:
-        r"""
-        Lottery ticket hypothesis algorithm (see page 2 of the paper):
+        r"""Lottery ticket hypothesis algorithm (see page 2 of the paper):
 
             1. Randomly initialize a neural network :math:`f(x; \theta_0)` (where :math:`\theta_0 \sim \mathcal{D}_\theta`).
             2. Train the network for :math:`j` iterations, arriving at parameters :math:`\theta_j`.
@@ -290,6 +294,7 @@ class ModelPruning(Callback):
         This function implements the step 4.
 
         The ``resample_parameters`` argument can be used to reset the parameters with a new :math:`\theta_z \sim \mathcal{D}_\theta`
+
         """  # noqa: E501
         assert self._original_layers is not None
         for d in self._original_layers.values():
@@ -299,14 +304,14 @@ class ModelPruning(Callback):
                 copy = deepcopy(copy)  # keep the original parameters
                 copy.reset_parameters()
             for i, name in names:
-                new, new_name = self._parameters_to_prune[i]
+                new, _ = self._parameters_to_prune[i]
                 self._copy_param(new, copy, name)
 
     def _apply_local_pruning(self, amount: float) -> None:
         for module, name in self._parameters_to_prune:
-            self.pruning_fn(module, name=name, amount=amount)
+            self.pruning_fn(module, name=name, amount=amount)  # type: ignore[call-arg]
 
-    def _resolve_global_kwargs(self, amount: float) -> Dict[str, Any]:
+    def _resolve_global_kwargs(self, amount: float) -> dict[str, Any]:
         self._global_kwargs["amount"] = amount
         params = set(inspect.signature(self.pruning_fn).parameters)
         params.discard("self")
@@ -318,7 +323,7 @@ class ModelPruning(Callback):
         )
 
     @staticmethod
-    def _get_pruned_stats(module: nn.Module, name: str) -> Tuple[int, int]:
+    def _get_pruned_stats(module: nn.Module, name: str) -> tuple[int, int]:
         attr = f"{name}_mask"
         if not hasattr(module, attr):
             return 0, 1
@@ -341,7 +346,7 @@ class ModelPruning(Callback):
 
     @rank_zero_only
     def _log_sparsity_stats(
-        self, prev: List[Tuple[int, int]], curr: List[Tuple[int, int]], amount: Union[int, float] = 0
+        self, prev: list[tuple[int, int]], curr: list[tuple[int, int]], amount: Union[int, float] = 0
     ) -> None:
         total_params = sum(p.numel() for layer, _ in self._parameters_to_prune for p in layer.parameters())
         prev_total_zeros = sum(zeros for zeros, _ in prev)
@@ -361,6 +366,7 @@ class ModelPruning(Callback):
                     f" {curr_mask_zeros} ({curr_mask_zeros / curr_mask_size:.2%})"
                 )
 
+    @override
     def setup(self, trainer: "pl.Trainer", pl_module: LightningModule, stage: str) -> None:
         parameters_to_prune = self.sanitize_parameters_to_prune(
             pl_module, self._parameters_to_prune, parameter_names=self._parameter_names
@@ -391,26 +397,29 @@ class ModelPruning(Callback):
         ):
             self.apply_lottery_ticket_hypothesis()
 
+    @override
     def on_train_epoch_end(self, trainer: "pl.Trainer", pl_module: LightningModule) -> None:
         if self._prune_on_train_epoch_end:
             rank_zero_debug("`ModelPruning.on_train_epoch_end`. Applying pruning")
             self._run_pruning(pl_module.current_epoch)
 
+    @override
     def on_validation_epoch_end(self, trainer: "pl.Trainer", pl_module: LightningModule) -> None:
         if not trainer.sanity_checking and not self._prune_on_train_epoch_end:
             rank_zero_debug("`ModelPruning.on_validation_epoch_end`. Applying pruning")
             self._run_pruning(pl_module.current_epoch)
 
+    @override
     def on_train_end(self, trainer: "pl.Trainer", pl_module: LightningModule) -> None:
         if self._make_pruning_permanent:
             rank_zero_debug("`ModelPruning.on_train_end`. Pruning is made permanent for this checkpoint")
             self.make_pruning_permanent(pl_module)
 
-    def _make_pruning_permanent_on_state_dict(self, pl_module: LightningModule) -> Dict[str, Any]:
+    def _make_pruning_permanent_on_state_dict(self, pl_module: LightningModule) -> dict[str, Any]:
         state_dict = pl_module.state_dict()
 
         # find the mask and the original weights.
-        map_pruned_params = {k.replace("_mask", "") for k in state_dict.keys() if k.endswith("_mask")}
+        map_pruned_params = {k.replace("_mask", "") for k in state_dict if k.endswith("_mask")}
         for tensor_name in map_pruned_params:
             orig = state_dict.pop(tensor_name + "_orig")
             mask = state_dict.pop(tensor_name + "_mask")
@@ -423,7 +432,8 @@ class ModelPruning(Callback):
 
         return apply_to_collection(state_dict, Tensor, move_to_cpu)
 
-    def on_save_checkpoint(self, trainer: "pl.Trainer", pl_module: LightningModule, checkpoint: Dict[str, Any]) -> None:
+    @override
+    def on_save_checkpoint(self, trainer: "pl.Trainer", pl_module: LightningModule, checkpoint: dict[str, Any]) -> None:
         if self._make_pruning_permanent:
             rank_zero_debug("`ModelPruning.on_save_checkpoint`. Pruning is made permanent for this checkpoint")
             # manually prune the weights so training can keep going with the same buffers
@@ -440,6 +450,7 @@ class ModelPruning(Callback):
             MisconfigurationException:
                 If ``parameters_to_prune`` doesn't exist in the model, or
                 if ``parameters_to_prune`` is neither a list nor a tuple.
+
         """
         parameters = parameter_names or ModelPruning.PARAMETER_NAMES
 
